#ifndef _LINUX_LVNODE_H_
#define _LINUX_LVNODE_H_

/**************************************************************************
 *
 * linux_lvnode.h
 *
 * LOMAC - Low Water-Mark Mandatory Access Control for Linux 
 * Copyright (C) 1999 NAI Labs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.  This program
 * is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.  You should have received a copy of the
 * GNU General Public License along with this program; if not, write
 * to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This is the public interface to linux_lvnode.c.
 *
 **************************************************************************/

#include <linux/fs.h>                 /* for struct inode */

#include "lomac_level.h"

typedef struct inode lvnode_t;        /* map linux inode to LOMAC vnode */

/* This structure encapsulates the information needed *
 * to uniquely identify an lvnode.                    */
typedef struct lvnode_id {
  unsigned long lvnode_number;
  unsigned int  device_number;
} lvnode_id_t;

/* This function is a predicate which indicates whether *
 * two lvnode_id_t's refer to the same lvnode or not.   */
static inline int lvnode_equals( const lvnode_id_t *p_id1,
				 const lvnode_id_t *p_id2 ) {

  return( ( p_id1->lvnode_number  == p_id2->lvnode_number  ) &&
	  ( p_id1->device_number  == p_id2->device_number  ) );

} /* lvnode_equal() */

/* This function fills in an lvnode_id_t structure for a given lvnode */
static inline void lvnode_to_id( lvnode_t *p_lvnode, lvnode_id_t *p_id ) {
  p_id->lvnode_number = p_lvnode->i_ino;
  p_id->device_number = p_lvnode->i_dev;
}

static inline lgid_t lvnode_gid( const lvnode_t *p_lvnode ) {
  return( p_lvnode->i_gid );
}

static inline int lvnode_is_nonlocal_socket( const lvnode_t *p_lvnode ) {
  return( ( p_lvnode->i_sock ) && 
	  ( p_lvnode->u.socket_i.ops->family != PF_UNIX ) );
}

static inline int lvnode_is_pipe( const lvnode_t *p_lvnode ) {

  return( ( p_lvnode->i_pipe ) &&                          /* is pipe */
	  ( p_lvnode->i_op != &fifo_inode_operations ) );  /* not FIFO */

}

lvnode_t *lvnode_from_path( const char * );
int lvnode_network_socket_to_level( lvnode_t *, level_t *, char ** );



#ifdef NOT_ANYMORE
static inline unsigned long lvnode_number( const lvnode_t *p_lvnode ) {
  return( p_lvnode->i_ino );
}

static inline unsigned int lvnode_device_number( const lvnode_t *p_lvnode ) {
  return( p_lvnode->i_dev );
}
#endif



#endif
