#ifndef _KERNEL_SYSCALLS_H_
#define _KERNEL_SYSCALLS_H_

/****************************************************************************
 *
 * linux_syscalls.h
 *
 * LOMAC - Low Water-Mark Mandatory Access Control for Linux 
 * Copyright (C) 1999 NAI Labs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.  This program
 * is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.  You should have received a copy of the
 * GNU General Public License along with this program; if not, write
 * to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 *
 *
 * Public interface to lomac_syscalls.c.  Creates storage for the addresses
 * of replaced system calls.
 *
 *
 ****************************************************************************/

#include <linux/ptrace.h>      /* for struct pt_regs for fork() */
#include <linux/uio.h>         /* for struct iovec * for readv() */

#include "kernel_intercept.h"

/* Create storage to hold pointers to the original kernel system call *
 * functions that we intend to intercept while Lomac is running.      */
SETUP_SYSCALL_STORAGE( read );
SETUP_SYSCALL_STORAGE( readv );
SETUP_SYSCALL_STORAGE( fchown );
SETUP_SYSCALL_STORAGE( chown );
SETUP_SYSCALL_STORAGE( open );
SETUP_SYSCALL_STORAGE( pipe );
SETUP_SYSCALL_STORAGE( creat );
SETUP_SYSCALL_STORAGE( socketcall );
SETUP_SYSCALL_STORAGE( kill );
SETUP_SYSCALL_STORAGE( write );
SETUP_SYSCALL_STORAGE( writev );
SETUP_SYSCALL_STORAGE( unlink );
SETUP_SYSCALL_STORAGE( rmdir );
SETUP_SYSCALL_STORAGE( truncate );
SETUP_SYSCALL_STORAGE( ftruncate );
SETUP_SYSCALL_STORAGE( reboot );
SETUP_SYSCALL_STORAGE( swapon );
SETUP_SYSCALL_STORAGE( swapoff );
SETUP_SYSCALL_STORAGE( setpgid );
SETUP_SYSCALL_STORAGE( setsid );

int lomac_fork( struct pt_regs );
int lomac_clone( struct pt_regs );
int lomac_exit( int );
int lomac_read( unsigned int, char *, int );
int lomac_readv( unsigned long, const struct iovec *, long );
int lomac_fchown( unsigned int, uid_t, gid_t );
int lomac_chown( const char *, uid_t, gid_t );
int lomac_open( const char *, int, int );
int lomac_pipe( unsigned long * );
int lomac_creat( const char *, int );
int lomac_socketcall( int, unsigned long * );
int lomac_kill( int, int );
int lomac_write( unsigned int, char *, unsigned int );
int lomac_writev( unsigned long, const struct iovec *, long );
int lomac_unlink( const char * );
int lomac_rmdir( const char * );
int lomac_truncate( const char *, unsigned long );
int lomac_ftruncate( unsigned int, unsigned long );
int lomac_reboot( int, int, int );
int lomac_swapon( const char *, int );
int lomac_swapoff( const char * );
int lomac_setpgid( pid_t, pid_t );
int lomac_setsid( void );

#endif
