#ifndef _LINUX_KERNEL_H_
#define _LINUX_KERNEL_H_

/**************************************************************************
 *
 * linux_kernel.h
 *
 * LOMAC - Low Water-Mark Mandatory Access Control for Linux 
 * Copyright (C) 1999 NAI Labs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.  This program
 * is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.  You should have received a copy of the
 * GNU General Public License along with this program; if not, write
 * to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 *
 * This file provides an OS-independent interface to the linux dynamic
 * memory allocation and logging functions.  
 * 
 * lprint needs some work to be truly portable.  panic may need some too.
 *
 **************************************************************************/

#include <linux/kernel.h>   /* for printk() and panic() */
#include <linux/limits.h>   /* for PATH_MAX */
#include <linux/mm.h>       /* for GFP_KERNEL */
#include <linux/malloc.h>   /* for kmalloc(), kfree() */

#ifdef PARANOID

/* paranoid versions of lmalloc and lfree attempt to keep    *
 * a count of current dynamic memory usage in `memory_used'. */

static unsigned long memory_used = 0;  /* a global count of mem alloc'd */

static void *lmalloc( unsigned int size ) {
  memory_used += size;  /* assumes alloc is going to succeed */
  return( kmalloc( size, GFP_KERNEL ) );
}

static void lfree( void *obj ) {
  memory_used -= sizeof( obj );
  kfree( obj );
}

#else

/* simple macros for non-paranoid memory management */
#define lmalloc( size )( kmalloc( (size), GFP_KERNEL ) )
#define lfree( p )( kfree( (p) ) )

#endif

#define lprint printk
#define lpanic panic
#define L_MAX_PATH_LENGTH PATH_MAX

#endif
