#ifndef _LINUX_LPROCESS_H_
#define _LINUX_LPROCESS_H_

/*************************************************************************
 *
 * linux_lprocess.h
 *
 * LOMAC - Low Water-Mark Mandatory Access Control for Linux 
 * Copyright (C) 1999 NAI Labs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.  This program
 * is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.  You should have received a copy of the
 * GNU General Public License along with this program; if not, write
 * to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This file defines the non-OS-dependent lprocess_t data type, which
 * hides the details of the linux task_struct data type from the rest
 * of LOMAC.
 *
 ************************************************************************/

#include <linux/sched.h>    /* for struct task_struct and `current' process */
#include <linux/mm.h>       /* for free_page used by putname() */

typedef struct task_struct lprocess_t;  /* map task_struct to LOMAC process */

static inline lpid_t lprocess_pid( lprocess_t *p_proc ) {
  return( p_proc->pid );
}

static inline lgid_t lprocess_gid( lprocess_t *p_proc ) {
  return( p_proc->gid );
}

static inline lgid_t lprocess_pgrp( lprocess_t *p_proc ) {
  return( p_proc->pgrp );
}

static inline lprocess_t *lprocess_next( lprocess_t *p_proc ) {
  return( p_proc->next_task );
}

#ifdef NOT_ANYMORE
static inline lprocess_t *lprocess_newest_child( lprocess_t *p_proc ) {
  return( p_proc->p_cptr );
}
#endif

static inline lprocess_t *lprocess_current( void ) {
  return( current );
}

static inline lprocess_t *lprocess_from_pid( lpid_t pid ) {
  
  lprocess_t *p = current;

  do {
    if( p->pid == pid ) {
      return( p );
    }
    p = p->next_task;
  } while( p != current );

  return( NULL );

}


/* lprocess_getname()
 *
 * in:     p_proc - pointer to process
 * out:    nothing
 * return: pointer to a dynamically alloc'd string containing the name
 *         of the binary `p_proc' is running.
 *
 * This function alloc's space for a string (name_s) and copies
 * argv[ 0 ] into it.  This is a bad way of determining the name
 * of the binary that `p_proc' is running, because the process
 * may have modified argv[ 0 ] or exec'd a different program by
 * the time this function is called.  In these cases, this function
 * will return the wrong answer.  We should eventually rig the
 * LOMAC binary format handler to store the correct names in the
 * PLM, and rely on that when doing verbose logging.
 *
 * Call lprocess_putname() when you're done with the name.
 */

static char *lprocess_getname( lprocess_t *p_proc ) {
  char *name_s;  /* buffer to hold name */
  
  name_s = getname( (char *)(current->mm->arg_start) );
  if( IS_ERR( name_s ) ) {
    panic( "LOMAC(linux_lprocess.h) unable to copy proc name "
	   "into kernelspace" );
  }
  return( name_s );
}

static inline void lprocess_putname( char *name_s ) {
  putname( name_s );
}

#endif
