/************************************************************************
 *
 * lomac_usi.c
 *
 * LOMAC - Low Water-Mark Mandatory Access Control for Linux 
 * Copyright (C) 1999 NAI Labs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.  This program
 * is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.  You should have received a copy of the
 * GNU General Public License along with this program; if not, write
 * to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 *
 *
 * These are the platform-independent functions which provide users
 * with information about the current state of LOMAC via the
 * platform-dependent lomac pseudodevice.
 *
 *
 *
 ************************************************************************/

#include "kernel_kernel.h"
#include "kernel_lgid.h"
#include "kernel_lpid.h"
#include "kernel_lvnode.h"

#include "lomac_level.h"
#include "lomac_fsolm.h"
#include "lomac_plm.h"
#include "lomac_usi.h"


/* ------------------------ Public Functions ----------------------- */

/* usi_get_pid_level()
 * 
 * in:     pid - pid whose level we want
 * out:    nothing
 * return: level of pid, or -1 if `pid' isn't currently in use
 *
 * The lomac pseudodevice driver can use this function to determine
 * the level of a process.
 *
 */

level_t usi_get_pid_level( lpid_t pid ) {

  level_t level;
  trust_t trust;

  if( plm_get( pid, &level, &trust ) ) {
    return( -1 );
  }

  return( level );

} /* usi_get_pid_level() */


/* usi_get_file_level()
 *
 * in:     pathname_s - absolute pathname whose level we want to know
 * out:    nothing
 * return: level of `pathname_s' or -1 if we don't have a level for it.
 *
 * The lomac pseudodevice driver can use this function to determine the
 * level of files.
 *
 */

level_t usi_get_file_level( char *pathname_s ) {

  lvnode_t *p_lvnode;  /* lvnode corresponding to `pathname_s', if any */

  if( !( p_lvnode = lvnode_from_path( pathname_s ) ) ) {
    return( -1 );
  }

  /* Make sure the FSOLM has an up-to-date entry for `p_lvnode'. */
  fsolm_set_for_existing_object( p_lvnode, 1 );

  return( fsolm_get( p_lvnode ) );

} /* usi_get_file_level() */


